import sys
import os
# 将项目根目录添加到 Python 路径
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
import pytest

from dotenv import load_dotenv
from llmada import BianXieAdapter, KimiAdapter, ArkAdapter
import os

load_dotenv()

class Test_Bianxie:
    @pytest.fixture
    def bianxie(self):
        return BianXieAdapter(api_key=os.getenv('BIANXIE_API_KEY'))

    def test_product(self,bianxie):
        bianxie.set_model('gpt-4o')
        result = bianxie.product(prompt='你好')
        print(result)
        assert type(result) == str

    def test_chat(self,bianxie):
        bianxie.set_model('gpt-4o')
        result = bianxie.chat(messages=[{'role': 'user', 'content': '你好'}])
        print(result)
        assert type(result) == str

    def test_no_api_key(self):
        bianxie = BianXieAdapter()
        bianxie.set_model('gpt-4o')
        result = bianxie.product(prompt='你好')
        print(result)
        assert type(result) == str

class Test_Kimi:

    @pytest.fixture
    def kimi(self):
        return KimiAdapter(api_key=os.getenv('MOONSHOT_API_KEY'))

    def test_product(self,kimi):
        kimi.set_model('moonshot-v1-128k')
        result = kimi.product(prompt='你好')
        print(result)
        assert type(result) == str


    def test_chat(self,kimi):
        kimi.set_model('moonshot-v1-128k')
        result = kimi.chat(messages=[{'role': 'user', 'content': '你好'}])
        print(result)
        assert type(result) == str

    def test_no_api_key(self):
        kimi = KimiAdapter()
        kimi.set_model('moonshot-v1-128k')
        result = kimi.product(prompt='你好')
        print(result)
        assert type(result) == str


class Test_Ark:

    @pytest.fixture
    def ark(self):
        return ArkAdapter(api_key=os.getenv('ARK_API_KEY'))

    def test_product(self,ark):
        ark.set_model('doubao-1-5-pro-256k-250115')
        result = ark.product(prompt='你好')
        print(result)
        assert type(result) == str


    def test_chat(self,ark):
        ark.set_model('doubao-1-5-pro-256k-250115')
        result = ark.chat(messages=[{'role': 'user', 'content': '你好'}])
        print(result)
        assert type(result) == str

    def test_no_api_key(self):
        ark = ArkAdapter()
        ark.set_model('doubao-1-5-pro-256k-250115')
        result = ark.product(prompt='你好')
        print(result)
        assert type(result) == str


"""
模型适配器
"""
from .client import OpenAIClient
from openai import OpenAI
import os

class ModelAdapter:
    def __init__(self):
        self.api_key = None
        self.model_name = None
        self.temperature = 0.7

    def set_model(self, model_name: str):
        self.model_name = model_name

    def product(self, prompt: str) -> str:
        raise NotImplementedError("This method should be implemented by subclasses")

    def chat(self, messages: list) -> str:
        raise NotImplementedError("This method should be implemented by subclasses")


class BianXieAdapter(ModelAdapter):
    """BianXie格式的适配器
    """
    def __init__(self, api_key: str = None, api_base: str = "https://api.bianxie.ai/v1/chat/completions"):
        """初始化

        Args:
            api_key (str): API key for authentication.
            api_base (str): Base URL for the API endpoint.
        """
        super().__init__()
        self.client = OpenAIClient(api_key=api_key or os.getenv('BIANXIE_API_KEY') , api_base=api_base)

    def get_model(self):
        return ["详见 看见"]

    def product(self, prompt: str) -> str:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            'model': self.model_name,
            'messages': [{'role': 'user', 'content': prompt}],
            'temperature': self.temperature
        }
        return self.client.request(data).get('choices')[0].get('message').get('content')

    def chat(self, messages: list) -> str:
        """Engage in a conversation with the model using a list of messages.

        Args:
            messages (list): A list of message dictionaries, each containing a role and content.

        Returns:
            str: The response generated by the model for the conversation.
        """
        data = {
            'model': self.model_name,
            'messages': messages,
            'temperature': self.temperature
        }
        return self.client.request(data).get('choices')[0].get('message').get('content')

class KimiAdapter(ModelAdapter):
    """Kimi格式的适配器

    """
    def __init__(self, api_key: str = None, api_base: str = "https://api.moonshot.cn/v1",):
        """初始化

        Args:
            api_key (str): API key for authentication.
            api_base (str): Base URL for the API endpoint.
        """
        super().__init__()
        
        self.client = OpenAI(api_key=api_key or os.getenv('MOONSHOT_API_KEY') , base_url=api_base)

    def get_model(self):
        return ["moonshot-v1-128k","moonshot-v1-128k","moonshot-v1-128k"]

    def product(self, prompt: str) -> str:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """

        data = {
            'model': self.model_name,
            'messages': [{'role': 'user', 'content': prompt}],
            'temperature': self.temperature
        }
        return self.client.chat.completions.create(**data).choices[0].message.content

    def chat(self, messages: list) -> str:
        """Engage in a conversation with the model using a list of messages.

        Args:
            messages (list): A list of message dictionaries, each containing a role and content.

        Returns:
            str: The response generated by the model for the conversation.
        """
        data = {
            'model': self.model_name,
            'messages': messages,
            'temperature': self.temperature
        }
        return self.client.chat.completions.create(**data).choices[0].message.content

class ArkAdapter(ModelAdapter):
    def __init__(self, api_key: str = None, api_base: str = None,):
        """初始化

        Args:
            api_key (str): API key for authentication.
            api_base (str): Base URL for the API endpoint.
        """
        super().__init__()

        from volcenginesdkarkruntime import Ark

        self.client = Ark(api_key=api_key or os.getenv('ARK_API_KEY'))

    def get_model(self):
        return ["doubao-1-5-pro-256k-250115"]

    def product(self, prompt: str) -> str:
        """Generate a response from the model based on a single prompt.

        Args:
            prompt (str): The input text prompt to generate a response for.

        Returns:
            str: The response generated by the model.
        """
        data = {
            'model': self.model_name,
            'messages': [{'role': 'user', 'content': prompt}],
            'temperature': self.temperature
        }
        return self.client.chat.completions.create(**data).choices[0].message.content

    def chat(self, messages: list) -> str:
        """Engage in a conversation with the model using a list of messages.

        Args:
            messages (list): A list of message dictionaries, each containing a role and content.

        Returns:
            str: The response generated by the model for the conversation.
        """
        data = {
            'model': self.model_name,
            'messages': messages,
            'temperature': self.temperature
        }
        return self.client.chat.completions.create(**data).choices[0].message.content


"""
client.py 用于管理client
"""
from llama_index.llms.openai import OpenAI
from llama_index.core import Settings
from llama_index.embeddings.openai import OpenAIEmbedding
from llama_index.core.tools import FunctionTool
from llama_index.core.llms import ChatMessage, MessageRole
from typing import Any, Dict, Optional
from typing import Dict, Any, Iterator
import openai

import requests

class OpenAIClient:
    """
    使用 openai 官方包对接 OpenAI API，支持简单对话和流式对话
    """
    def __init__(self, api_key: str, api_base: str = "https://api.bianxie.ai/v1/chat/completions"):
        """
        初始化 OpenAI 客户端
        """
        self.api_key = api_key
        self.api_base = api_base
        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {api_key}'
        }

    def request(self, params: dict) -> dict:
        """
        简单对话：直接调用 OpenAI API 并返回完整响应
        """

        try:
            response = requests.post(self.api_base, headers=self.headers, json=params)
            return response.json()
        except Exception as e:
            raise Exception(f"API request failed: {e}")

